package api

import (
	"box-cost/db/model"
	"fmt"
	"strconv"

	"github.com/xuri/excelize/v2"
)

type ProduceBillExcel struct {
	Offset int
	Row    int

	Title string //标题

	Excel *excelize.File

	SheetName string

	AlignCenterStyle int

	Content    *model.ProduceBill
	Signatures []*model.Signature
}

func (b *ProduceBillExcel) drawTitle() error {
	// 设置外边距
	marginLeft := excelize.PageMarginLeft(0.15)
	b.Excel.SetPageMargins(b.SheetName, marginLeft)
	tileIndex := b.Offset + 1
	startCell := fmt.Sprintf("A%d", tileIndex)
	err := b.Excel.MergeCell(b.SheetName, startCell, fmt.Sprintf("L%d", tileIndex))
	if err != nil {
		return err
	}

	style, err := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "center", Vertical: "center"},
		Font:      &excelize.Font{Bold: true, Size: 18}})
	if err != nil {
		return err
	}
	err = b.Excel.SetCellStyle(b.SheetName, startCell, startCell, style)
	if err != nil {
		return err
	}
	b.Excel.SetRowHeight(b.SheetName, tileIndex, 23)
	b.Excel.SetCellValue(b.SheetName, startCell, b.Title)
	return nil
}

func (b *ProduceBillExcel) drawSubTitles() error {
	row := b.Offset + 2

	styleLeft, err := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "left", Vertical: "center"},
		Font:      &excelize.Font{Size: 11}})
	if err != nil {
		return err
	}
	styleRight, err := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "right", Vertical: "center"},
		Font:      &excelize.Font{Size: 11}})
	if err != nil {
		return err
	}

	var drawLeft = func(rowIndex int, value string) error {
		//左边1
		left1Cell := fmt.Sprintf("A%d", rowIndex)
		err = b.Excel.MergeCell(b.SheetName, left1Cell, fmt.Sprintf("E%d", rowIndex))
		if err != nil {
			return err
		}
		err = b.Excel.SetCellStyle(b.SheetName, left1Cell, left1Cell, styleLeft)
		if err != nil {
			return err
		}
		b.Excel.SetCellValue(b.SheetName, left1Cell, value)
		return nil
	}

	var drawRight = func(rowIndex int, value string) error {
		right1Cell := fmt.Sprintf("F%d", rowIndex)
		err = b.Excel.MergeCell(b.SheetName, right1Cell, fmt.Sprintf("L%d", rowIndex))
		if err != nil {
			return err
		}
		err = b.Excel.SetCellStyle(b.SheetName, right1Cell, right1Cell, styleRight)
		if err != nil {
			return err
		}
		b.Excel.SetCellValue(b.SheetName, right1Cell, value)
		return nil
	}

	//第一行
	drawLeft(row, "类别："+b.Content.Type)
	drawRight(row, "单号："+b.Content.SerialNumber)
	b.Excel.SetRowHeight(b.SheetName, row, 21)

	//第二行
	drawLeft(row+1, "供应商名称："+b.Content.Supplier)
	timeformat := b.Content.CreateTime.Local().Format("2006年01月02号 15:04:05")
	drawRight(row+1, "下单时间："+timeformat)
	b.Excel.SetRowHeight(b.SheetName, row+1, 21)

	//第三行
	drawLeft(row+2, "产品名称："+b.Content.ProductName)
	status := ""
	if b.Content.Status == "complete" {
		status = fmt.Sprintf("已完成(%d)", b.Content.ConfirmCount)
	}
	if b.Content.Status == "created" {
		status = "进行中"
	}
	drawRight(row+2, "状态："+status)
	b.Excel.SetRowHeight(b.SheetName, row+2, 21)

	return nil
}

func (b *ProduceBillExcel) drawTableTitle() error {
	row := b.Offset + 5

	//A加工项目 B规格(克) 尺寸C-D 数量E 单价F-G 交货时间H 备注I
	var drawCol = func(prefix string, value string) error {
		left1Cell := fmt.Sprintf("%s%d", prefix, row)
		left2Cell := fmt.Sprintf("%s%d", prefix, row+1)

		err := b.Excel.MergeCell(b.SheetName, left1Cell, left2Cell)
		if err != nil {
			return err
		}
		err = b.Excel.SetCellStyle(b.SheetName, left1Cell, left2Cell, b.AlignCenterStyle)
		if err != nil {
			return err
		}

		return b.Excel.SetCellValue(b.SheetName, left1Cell, value)
	}

	var drawCol2 = func(prefix string, value1 string, value2 string) error {

		left1Cell := fmt.Sprintf("%s%d", prefix, row)
		left2Cell := fmt.Sprintf("%s%d", prefix, row+1)
		err := b.Excel.SetCellStyle(b.SheetName, left1Cell, left2Cell, b.AlignCenterStyle)
		if err != nil {
			return err
		}
		b.Excel.SetCellValue(b.SheetName, left1Cell, value1)
		b.Excel.SetCellValue(b.SheetName, left2Cell, value2)
		return nil
	}

	drawCol("A", "加工项目")
	drawCol("B", "规格")
	drawCol("C", "纸张")
	drawCol("D", "来纸尺寸")
	drawCol("E", "印刷尺寸")
	drawCol("F", "数量")
	drawCol("G", "完成数量")
	drawCol2("H", "单价", "元/张")
	drawCol("I", "预算金额")
	drawCol("J", "实际金额")
	drawCol("K", "交货时间")
	drawCol("L", "备注")

	return nil
}

func (b *ProduceBillExcel) drawTableContent() error {
	row := b.Offset + 7
	b.Row = row
	var DrawRow = func(rowIndex int, values ...string) {
		charas := []string{"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L"}
		for i, c := range charas {
			v := ""
			if i < len(values) {
				v = values[i]
			}
			b.Excel.SetCellValue(b.SheetName, fmt.Sprintf("%s%d", c, rowIndex), v)
			val2Cel := fmt.Sprintf("%s%d", c, rowIndex)
			b.Excel.SetCellStyle(b.SheetName, val2Cel, val2Cel, b.AlignCenterStyle)

			b.Excel.SetRowHeight(b.SheetName, rowIndex, 21)
		}
	}

	produces := b.Content.Produces
	if len(produces) > 0 {
		for _, produce := range produces {

			confirmCount := "-"
			realAmount := "-"
			price, err := strconv.ParseFloat(produce.Price, 64)
			if err != nil {
				price = 0.00
			}

			// 预算金额
			budgetAmount := fmt.Sprintf("%.2f", float64(produce.Count)*price)
			b.FormatToEmpty(&budgetAmount)

			if b.Content.Status == "complete" {
				// 实际完成数
				confirmCount = fmt.Sprintf("%d", b.Content.ConfirmCount)
				b.FormatToEmpty(&confirmCount)

				// 实际金额
				realAmount = fmt.Sprintf("%.2f", float64(b.Content.ConfirmCount)*price)
				b.FormatToEmpty(&realAmount)
			}

			deliveryTime := produce.DeliveryTime.Local().Format("2006-01-02")
			DrawRow(row, produce.Name, produce.Norm, produce.Paper, produce.PaperSize, produce.PrintSize, fmt.Sprintf("%d", produce.Count), confirmCount, produce.Price, budgetAmount, realAmount, deliveryTime, produce.Remark)
			row++
			b.Row++
		}
	}

	return nil
}

func (b *ProduceBillExcel) drawTableFooter() error {
	// row := b.Offset + 8
	// b.Row++
	row := b.Row

	left1Cell := fmt.Sprintf("A%d", row)

	border := []excelize.Border{
		{Type: "top", Style: 1, Color: "000000"},
		{Type: "left", Style: 1, Color: "000000"},
		{Type: "right", Style: 1, Color: "000000"},
		{Type: "bottom", Style: 1, Color: "000000"},
	}

	styleLeft, _ := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "left", Vertical: "center"},
		Border:    border,
	})

	b.Excel.SetCellStyle(b.SheetName, left1Cell, left1Cell, styleLeft)
	b.Excel.SetCellValue(b.SheetName, left1Cell, "送货地址")

	addCel := fmt.Sprintf("B%d", row)

	b.Excel.MergeCell(b.SheetName, addCel, fmt.Sprintf("E%d", row))
	b.Excel.SetCellStyle(b.SheetName, addCel, fmt.Sprintf("E%d", row), styleLeft)
	b.Excel.SetCellValue(b.SheetName, addCel, b.Content.SendTo)

	sureCel := fmt.Sprintf("F%d", row)
	b.Excel.MergeCell(b.SheetName, sureCel, fmt.Sprintf("L%d", row))
	b.Excel.SetCellStyle(b.SheetName, sureCel, fmt.Sprintf("L%d", row), styleLeft)
	b.Excel.SetCellValue(b.SheetName, sureCel, "供应商签字：")

	b.Excel.SetRowHeight(b.SheetName, row, 21)

	return nil
}

func (b *ProduceBillExcel) drawTableSignature() error {
	// row := b.Offset + 11
	b.Row += 2
	row := b.Row

	style1, _ := b.Excel.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "center", Vertical: "center"},
		// Border:    border,
	})

	fontCell := fmt.Sprintf("H%d", row)
	imageCell1 := fmt.Sprintf("I%d", row)
	imageCell2 := fmt.Sprintf("K%d", row)
	b.Excel.MergeCell(b.SheetName, fontCell, fmt.Sprintf("H%d", row+2))
	b.Excel.SetCellStyle(b.SheetName, fontCell, fontCell, style1)
	b.Excel.SetCellValue(b.SheetName, fontCell, "领导签字：")

	// 签字图片1 I10-J12
	b.Excel.MergeCell(b.SheetName, imageCell1, fmt.Sprintf("J%d", row+2))
	b.Excel.SetCellStyle(b.SheetName, imageCell1, imageCell1, style1)
	b.Excel.SetCellValue(b.SheetName, imageCell1, "")

	// 签字图片2 K10-L12
	b.Excel.MergeCell(b.SheetName, imageCell2, fmt.Sprintf("L%d", row+2))
	b.Excel.SetCellStyle(b.SheetName, imageCell2, imageCell1, style1)
	b.Excel.SetCellValue(b.SheetName, imageCell2, "")

	// 状态为已审核时，签字
	if b.Content.Reviewed == 1 {
		imageCells := []string{imageCell1, imageCell2}
		if len(b.Signatures) > 0 {
			for k, sign := range b.Signatures {
				b.Excel.AddPicture(b.SheetName, imageCells[k], sign.Url, sign.Format)

			}
		}
	}

	b.Excel.SetRowHeight(b.SheetName, row, 21)

	return nil
}

func (b *ProduceBillExcel) Draws() {
	b.drawTitle()
	b.drawSubTitles()
	b.drawTableTitle()
	b.drawTableContent()
	b.drawTableFooter()
	b.drawTableSignature()
}

func NewProduceBill(f *excelize.File) *ProduceBillExcel {

	border := []excelize.Border{
		{Type: "top", Style: 1, Color: "000000"},
		{Type: "left", Style: 1, Color: "000000"},
		{Type: "right", Style: 1, Color: "000000"},
		{Type: "bottom", Style: 1, Color: "000000"},
	}

	styleLeft, _ := f.NewStyle(&excelize.Style{
		Alignment: &excelize.Alignment{Horizontal: "center", Vertical: "center"},
		Border:    border,
	})

	b := &ProduceBillExcel{
		Title:            "中鱼互动加工单",
		SheetName:        "Sheet1",
		Excel:            f,
		Offset:           0,
		AlignCenterStyle: styleLeft,
		Signatures:       make([]*model.Signature, 0),
	}

	// f.SetColWidth(b.SheetName, "A", "A", 17)
	// f.SetColWidth(b.SheetName, "B", "B", 13)
	// f.SetColWidth(b.SheetName, "C", "G", 9.5)
	// f.SetColWidth(b.SheetName, "H", "I", 12)

	f.SetColWidth(b.SheetName, "A", "A", 12)
	f.SetColWidth(b.SheetName, "B", "K", 9.5)
	// f.SetColWidth(b.SheetName, "I", "J", 8)
	f.SetColWidth(b.SheetName, "L", "L", 10)
	f.SetPageMargins(b.SheetName, excelize.PageMarginTop(0), excelize.PageMarginLeft(0), excelize.PageMarginRight(0))

	return b
}

func (b *ProduceBillExcel) FormatToEmpty(str *string) {
	if *str == "0" || *str == "0.00" {
		*str = ""
	}

}
